class TreeChordFlag3(object):
    """
    These flags would be processed after updating type, dots and grouping beams (before creating notes)
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def __deepcopy__(self, memodict={}):
        return self.__class__()



import os

from quicktions import Fraction

from musicscore.basic_functions import dToX
from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicxmlunittest import XMLTestCase

path = os.path.abspath(__file__).split('.')[0]


def _generate_simple_format():
    unit = Fraction(1, 2)
    quarter_durations = [x * unit for x in range(1, 8)]
    intervals = list(range(1, 7))
    midis = dToX(intervals, first_element=60)
    output = SimpleFormat(quarter_durations=quarter_durations, midis=midis)
    return output


class Test(XMLTestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()
        self.sf = _generate_simple_format()

    def test_1(self):
        xml_path = path + '_test_1.xml'
        self.sf.to_stream_voice().add_to_score(self.score)

        def _change_quarter_duration(chord, factor):
            chord.quarter_duration *= factor

        self.sf.change_chords(lambda chord: _change_quarter_duration(chord, 2))
        self.sf.to_stream_voice().add_to_score(self.score, staff_number=2)
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_2(self):
        xml_path = path + '_test_2.xml'
        self.sf.to_stream_voice().add_to_score(self.score)

        def _change_quarter_duration(chord, factor):
            chord.quarter_duration *= factor

        self.sf.change_chords(lambda chord: _change_quarter_duration(chord, 2) if int(
            chord.quarter_duration) != chord.quarter_duration else None)
        self.sf.to_stream_voice().add_to_score(self.score, staff_number=2)
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)



from pathlib import Path

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicxmlunittest import XMLTestCase

path = Path(__file__)


class Test(XMLTestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_single_syllabic(self):
        xml_path = path.parent.joinpath(path.stem + '_single_syllabic.xml')
        sf = SimpleFormat(quarter_durations=4)
        sf.chords[0].add_lyric('one', syllabic='single')
        sf.to_stream_voice().add_to_score(self.score)
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_multi_syllabic(self):
        xml_path = path.parent.joinpath(path.stem + '_multi_syllabic.xml')
        sf = SimpleFormat(quarter_durations=[1, 1, 1, 1])
        sf.chords[0].add_lyric('non', syllabic='begin')
        sf.chords[1].add_lyric('syl', syllabic='middle')
        sf.chords[2].add_lyric('la', syllabic='middle')
        sf.chords[3].add_lyric('bic', syllabic='end')
        sf.to_stream_voice().add_to_score(self.score)
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_extend(self):
        xml_path = path.parent.joinpath(path.stem + '_extend.xml')
        sf = SimpleFormat(quarter_durations=[1, 1, 1, 1])
        sf.chords[0].add_lyric('one', syllabic='single', extend='start')
        # sf.chords[1].add_lyric(extend='continue')
        sf.chords[3].add_lyric('two', syllabic='single')
        sf.to_stream_voice().add_to_score(self.score)
        self.score.finish()
        partwise = self.score.to_partwise()
        partwise.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_melismatic(self):
        xml_path = path.parent.joinpath(path.stem + '_melismatic.xml')
        sf = SimpleFormat(quarter_durations=[0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 1])
        sf.chords[0].add_lyric('mar', syllabic='begin')
        sf.chords[3].add_lyric('a', syllabic='middle')
        sf.chords[-1].add_lyric('thon', syllabic='end')
        sf.to_stream_voice().add_to_score(self.score)
        self.score.finish()
        partwise = self.score.to_partwise()
        partwise.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_simple(self):
        xml_path = path.parent.joinpath(path.stem + '_simple.xml')
        sf = SimpleFormat(quarter_durations=[0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 1])
        sf.chords[0].add_lyric(1)
        sf.chords[3].add_lyric(2)
        sf.chords[-1].add_lyric(3)
        sf.to_stream_voice().add_to_score(self.score)
        self.score.finish()
        partwise = self.score.to_partwise()
        partwise.write(xml_path)
        self.assertCompareFiles(xml_path)




from unittest import TestCase

from musicscore.musicxml.groups.musicdata import Attributes
from musicscore.musicxml.types.complextypes.attributes import Clef
from musicscore.musicxml.types.complextypes.clef import Sign


class TestTime(TestCase):

    def setUp(self):
        self.attributes = Attributes()

    def test_1(self):
        clef_1 = Clef(number=1)
        clef_1.add_child(Sign('G'))
        self.attributes.add_child(clef_1)

        clef_2 = Clef(number=2)
        clef_2.add_child(Sign('F'))
        self.attributes.add_child(clef_2)
        expected ='''<attributes>
  <clef number="1">
    <sign>G</sign>
  </clef>
  <clef number="2">
    <sign>F</sign>
  </clef>
</attributes>
'''
        actual = self.attributes.to_string()
        self.assertEqual(expected, actual)



from unittest import TestCase
import os

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from tests.score_templates.xml_test_score import TestScore

path = os.path.abspath(__file__).split('.')[0]


class Test(TestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()
        sf = SimpleFormat(quarter_durations=[1, 1])
        v = sf.to_stream_voice(1)
        v.add_to_score(self.score)

    def test_1(self):
        self.score.add_title('test title', font_family='DejaVu Sans')
        self.score.add_subtitle('test subtitle', font_family='DejaVu Sans')
        result_path = path + '_test_1'
        self.score.write(path=result_path)
        TestScore().assert_template(result_path=result_path)

    def test_2(self):
        self.score.page_style.format = 'landscape'
        self.score.add_title('test title', font_family='DejaVu Sans')
        self.score.add_subtitle('test subtitle', font_family='DejaVu Sans')

        result_path = path + '_test_2'
        self.score.write(path=result_path)
        # TestScore().assert_template(result_path=result_path)



import os

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicxmlunittest import XMLTestCase

path = os.path.abspath(__file__).split('.')[0]


class TestAddBracket(XMLTestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_arrow(self):
        xml_path = path + '_test_arrow.xml'

        sf = SimpleFormat(quarter_durations=5 * [2.25])
        sf.chords[0].add_bracket(type='start', line_end='none', relative_x=50)
        sf.chords[1].add_bracket(type='continue', line_end='none')
        sf.chords[2].add_bracket(type='stop', line_end='arrow', relative_x=-30, relative_y=20)

        sf.to_stream_voice().add_to_score(self.score)
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    # def test_dashed_arrow(self):
    #     xml_path = path + '_test_dashed_arrow.xml'
    #
    #     sf = SimpleFormat(quarter_durations=5 * [2.25])
    #     sf.chords[0].add_bracket(type='start', line_end='none', line_type='dashed', relative_x=50)
    #     sf.chords[1].add_bracket(type='continue', line_type='dashed', line_end='none')
    #     sf.chords[2].add_bracket(type='stop', line_end='arrow', line_type='dashed', relative_x=-30, relative_y=20)
    #
    #     sf.to_stream_voice().add_to_score(self.score)
    #     self.score.write(xml_path)
    #     self.assertCompareFiles(xml_path)



import os
from unittest import TestCase

from musicscore.musictree.treechord import TreeChord

path = str(os.path.abspath(__file__).split('.')[0])


class Test(TestCase):

    def test_1(self):
        chord = TreeChord(quarter_duration=4, midis=[60])
        self.assertIsNone(chord.staff_number)

    def test_2(self):
        chord = TreeChord(quarter_duration=4, midis=[60])
        chord.staff_number = 1
        expected = 1
        actual = chord.staff_number
        self.assertEqual(expected, actual)

    def test_3(self):
        chord = TreeChord(quarter_duration=4, midis=[60])
        chord.staff_number = 1
        chord.staff_number = 2
        expected = 2
        actual = chord.staff_number
        self.assertEqual(expected, actual)



from pathlib import Path

from quicktions import Fraction

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treechordflags3 import TreeChordFlag3
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicscore.musicxml.elements.note import Cue, Type
from musicxmlunittest import XMLTestCase

path = Path(__file__)


class CueTypeFlag(TreeChordFlag3):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def implement(self, chord):
        t = chord.get_children_by_type(Type)[0]
        t.size = 'cue'
        return [chord]


class TestCue(XMLTestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_add_child(self):
        xml_path = path.parent.joinpath(path.stem + '_add_child.xml')
        sf = SimpleFormat(quarter_durations=16 * [Fraction(1, 8)])
        for chord in sf.chords[1:-1]:
            chord.add_child(Cue())
        sf.to_stream_voice().add_to_score(self.score)
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_add_flag(self):
        xml_path = path.parent.joinpath(path.stem + '_add_flag.xml')
        sf = SimpleFormat(quarter_durations=16 * [Fraction(1, 8)])
        for chord in sf.chords[1:-1]:
            chord.add_flag(CueTypeFlag())
        sf.to_stream_voice().add_to_score(self.score)
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)



import os
from unittest import TestCase

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from tests.score_templates.xml_test_score import TestScore

path = str(os.path.abspath(__file__).split('.')[0])


class Test(TestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_1(self):
        sf = SimpleFormat(quarter_durations=[4, 4, 4, 4, 4])
        v = sf.to_stream_voice(1)
        v.add_to_score(self.score)
        self.score.get_measure(3).add_page_break()

        result_path = path + '_test_1'
        self.score.write(path=result_path)
        TestScore().assert_template(result_path=result_path)

    def test_2(self):
        sf = SimpleFormat(quarter_durations=[4, 4, 4, 4, 4])
        v = sf.to_stream_voice(1)
        v.add_to_score(self.score, 1)
        sf = SimpleFormat(quarter_durations=[4, 4, 4, 4, 4])
        v = sf.to_stream_voice(1)
        v.add_to_score(self.score, 2)
        self.score.get_measure(3).add_page_break()

        result_path = path + '_test_2'
        self.score.write(path=result_path)
        TestScore().assert_template(result_path=result_path)



import os

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicscore.musicxml.elements.note import Stem
from musicxmlunittest import XMLTestCase

path = str(os.path.abspath(__file__).split('.')[0])


class Test(XMLTestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_1(self):
        sf = SimpleFormat(quarter_durations=[1, 5], midis=[60, 60])
        for chord in sf.chords:
            chord.add_child(Stem('down'))
        sf.to_stream_voice().add_to_score(self.score)
        xml_path = path + '_test_1.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)



from musicscore.musicxml.attributes.font import Font
from musicscore.musicxml.types.complextypes.complextype import ComplexType
from musicscore.musicxml.types.simple_type import Token


class ComplexTypeLyricFont(ComplexType, Font):
    """The lyric-font type specifies the default font for a particular name and number of lyric.
    """

    def __init__(self, tag, *args, **kwargs):
        super().__init__(tag=tag, *args, **kwargs)

    @property
    def number(self):
        return self.get_attribute('number')

    @number.setter
    def number(self, value):
        if value is None:
            self.remove_attribute('number')
        else:
            Token(value)
            self._ATTRIBUTES.insert(0, 'number')
            self.set_attribute('number', value)

    @property
    def name(self):
        return self.get_attribute('name')

    @name.setter
    def name(self, value):
        if value is None:
            self.remove_attribute('name')
        else:
            Token(value)
            self._ATTRIBUTES.insert(0, 'name')
            self.set_attribute('name', value)



import os

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicxmlunittest import XMLTestCase

path = str(os.path.abspath(__file__).split('.')[0])


class Test(XMLTestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_1(self):
        xml_path = path + '_test_1.xml'
        sf_1 = SimpleFormat(quarter_durations=[1, 2, 3, 2, 1], midis=[60, (60, 62), (64, 66, 71), 72, 73])
        sf_1.to_stream_voice().add_to_score(self.score)
        sf_2 = SimpleFormat(quarter_durations=[0.5, 1, 1.5, 2, 3], midis=[0, 69, (72, 73), (58, 60, 65, 71), 80])
        sf_2.to_stream_voice().add_to_score(self.score, staff_number=2)
        sf_3 = SimpleFormat.sum(sf_1, sf_2)
        sf_3.to_stream_voice().add_to_score(self.score, staff_number=3)
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_2(self):
        xml_path = path + '_test_2.xml'
        sf_1 = SimpleFormat(quarter_durations=[1, 2, 3, 2, 1], midis=[60, (60, 62), (64, 66, 71), 72, 73])
        sf_1.to_stream_voice().add_to_score(self.score)
        sf_2 = SimpleFormat(quarter_durations=[0.5, 1, 1.5, 2, 3], midis=[0, 69, (72, 73), (58, 60, 65, 71), 80])
        sf_2.to_stream_voice().add_to_score(self.score, staff_number=2)
        sf_3 = SimpleFormat.sum(sf_1, sf_2, no_doubles=True)
        sf_3.to_stream_voice().add_to_score(self.score, staff_number=3)
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_3(self):
        xml_path = path + '_test_3.xml'
        sf_1 = SimpleFormat(quarter_durations=[1, 2, 3, 2, 1], midis=[60, (60, 62), (64, 66, 71), 72, 73])
        sf_2 = SimpleFormat(quarter_durations=[0.5, 1, 1.5, 2, 3], midis=[0, 69, (72, 73), (58, 60, 65, 71), 80])
        sf_3 = SimpleFormat(quarter_durations=[1.5, 1.5, 1.5], midis=[(55, 58), 0, 57])

        sfs = [sf_1, sf_2, sf_3]

        for index, sf in enumerate(sfs):
            sf.to_stream_voice().add_to_score(self.score, staff_number=index + 1)

        sum_sf = SimpleFormat.sum(*sfs, no_doubles=True)
        sum_sf.to_stream_voice().add_to_score(self.score, staff_number=len(sfs) + 1)

        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)



from musicscore.musictree.treechord import TreeNote, TreeChord
from musicscore.musicxml.elements.fullnote import Pitch
from musicscore.musicxml.elements.note import Duration
from musicxmlunittest import XMLTestCase


class Test(XMLTestCase):
    def setUp(self) -> None:
        self.chord = TreeChord(quarter_duration=2)
        self.note = TreeNote(parent_chord=self.chord)

    def test_1(self):
        expected = self.chord.quarter_duration
        actual = self.note.quarter_duration
        self.assertEqual(actual, expected)

    def test_2(self):
        # accidental
        self.note.accidental.show = False
        self.note.add_child(Duration(int(self.note.quarter_duration)))
        self.note.event = Pitch(step='A', alter=-1.5)
        self.note.accidental.show = True
        result = '''<note>
  <pitch>
    <step>A</step>
    <alter>-1.5</alter>
    <octave>4</octave>
  </pitch>
  <duration>2</duration>
  <accidental>three-quarters-flat</accidental>
</note>
'''
        self.assertEqual(self.note.to_string(), result)



import os

from quicktions import Fraction

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treechord import TreeChord
from musicscore.musictree.treeclef import BASS_CLEF
from musicscore.musictree.treemeasure import TreeMeasure
from musicscore.musictree.treepart import TreePart
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicxmlunittest import XMLTestCase

path = str(os.path.abspath(__file__).split('.')[0])


class Test(XMLTestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_1(self):
        measure = self.score.add_measure()
        self.score.add_part()
        chord = TreeChord(quarter_duration=4, midis=[60])
        measure.get_part(1).add_chord(chord)
        measure.get_part(1).staves = 2
        chord.staff_number = 1
        xml_path = path + '_test_1.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_2(self):
        measure = self.score.add_measure()
        self.score.add_part()
        chord = TreeChord(quarter_duration=4, midis=[60])
        measure.get_part(1).add_chord(chord)
        chord.staff_number = 2
        measure.get_part(1).staves = 2
        xml_path = path + '_test_2.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_3(self):
        sf = SimpleFormat(quarter_durations=[0.25, 2.25, 0.5, 1, 2, Fraction(1, 3), Fraction(4, 3), Fraction(1, 3), 1])
        for index, chord in enumerate(sf.chords):
            if index in [1, 6]:
                chord.staff_number = 2
        sf.to_stream_voice(1).add_to_score(self.score, part_number=1)
        all_parts = [part for m in self.score.get_children_by_type(TreeMeasure) for part in
                     m.get_children_by_type(TreePart)]
        for part in all_parts:
            part.staves = 2

        xml_path = path + '_test_3.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_4(self):
        sf = SimpleFormat(4)
        sf.to_stream_voice(1).add_to_score(self.score, part_number=1)
        score_part = self.score.get_score_parts()[0]
        score_part.number_of_staves = 2
        xml_path = path + '_test_4.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_5(self):
        sf = SimpleFormat(4)
        sf.to_stream_voice(1).add_to_score(self.score, part_number=1)
        score_part = self.score.get_score_parts()[0]
        score_part.number_of_staves = 2
        clef = BASS_CLEF.__deepcopy__()
        clef.number = 2
        self.score.get_measure(1).get_part(1).add_clef(clef)
        xml_path = path + '_test_5.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_6(self):
        r_sf = SimpleFormat(quarter_durations=[4])
        l_sf = SimpleFormat(quarter_durations=[4], midis=[60])
        r_sf.to_stream_voice().add_to_score(self.score, part_number=1, staff_number=1)
        l_sf.to_stream_voice().add_to_score(self.score, part_number=1, staff_number=2)
        xml_path = path + '_test_6.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_7(self):
        r_sf = SimpleFormat(
            quarter_durations=[0.25, 2.25, 0.5, 1, 2, Fraction(1, 3), Fraction(4, 3), Fraction(1, 3), 1])
        l_sf = SimpleFormat(quarter_durations=[4, 5], midis=[72, 75])
        r_sf.to_stream_voice().add_to_score(self.score, part_number=1, staff_number=1)
        l_sf.to_stream_voice().add_to_score(self.score, part_number=1, staff_number=2)
        xml_path = path + '_test_7.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_8(self):
        r_sf = SimpleFormat(
            quarter_durations=[0.25, 2.25, 0.5, 1, 2, Fraction(1, 3), Fraction(4, 3), Fraction(1, 3), 1])
        l_sf = SimpleFormat(quarter_durations=[4, 5], midis=[50, 55])
        r_sf.to_stream_voice().add_to_score(self.score, part_number=1, staff_number=1)
        l_sf.to_stream_voice().add_to_score(self.score, part_number=1, staff_number=2)
        clef = BASS_CLEF.__deepcopy__()
        clef.number = 2
        self.score.get_measure(1).get_part(1).add_clef(clef)
        xml_path = path + '_test_8.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_9(self):
        r_sf = SimpleFormat(
            quarter_durations=[0.25, 2.25, 0.5, 1, 2, Fraction(1, 3), Fraction(4, 3), Fraction(1, 3), 1],
            midis=[60, 61, 62, 63])
        l_sf = SimpleFormat(quarter_durations=[4, 5], midis=[50, 55])
        r_sf.chords[1].manual_staff_number = 2
        r_sf.to_stream_voice().add_to_score(self.score, part_number=1, staff_number=1)
        l_sf.to_stream_voice().add_to_score(self.score, part_number=1, staff_number=2)
        clef = BASS_CLEF.__deepcopy__()
        clef.number = 2
        self.score.get_measure(1).get_part(1).add_clef(clef)
        xml_path = path + '_test_9.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_10(self):
        # chord clef
        r_sf = SimpleFormat(
            quarter_durations=[4, 4],
            midis=[60, 61])
        l_sf = SimpleFormat(quarter_durations=[4, 4], midis=[50, 55])
        l_sf.chords[0].add_clef(BASS_CLEF)
        r_sf.to_stream_voice().add_to_score(self.score, part_number=1, staff_number=1)
        l_sf.to_stream_voice().add_to_score(self.score, part_number=1, staff_number=2)
        xml_path = path + '_test_10.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_11(self):
        r_sf = SimpleFormat(
            quarter_durations=[0.25, 2.25, 0.5, 1, 2, Fraction(1, 3), Fraction(4, 3), Fraction(1, 3), 1],
            midis=[60, 61, 54, 63])
        l_sf = SimpleFormat(quarter_durations=[4, 5], midis=[72, 53])
        r_sf.chords[1].manual_staff_number = 2
        r_sf.auto_clef()
        l_sf.auto_clef()
        r_sf.to_stream_voice().add_to_score(self.score, part_number=1, staff_number=1)
        l_sf.to_stream_voice().add_to_score(self.score, part_number=1, staff_number=2)

        xml_path = path + '_test_11.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_12(self):
        r_sf_2 = SimpleFormat(
            quarter_durations=[0.25, 2.25, 0.5, 1, 2, Fraction(1, 3), Fraction(4, 3), Fraction(1, 3), 1],
            midis=[60, 61, 54, 63])
        r_sf_1 = SimpleFormat(
            quarter_durations=[0.25, 2.25, 0.5, 1, 2, Fraction(1, 3), Fraction(4, 3), Fraction(1, 3), 1],
            midis=[70, 71, 64, 73])
        l_sf_1 = SimpleFormat(quarter_durations=[4, 5], midis=[50, 35])
        l_sf_2 = SimpleFormat(quarter_durations=[5, 4], midis=[44, 60])
        r_sf_2.auto_clef()
        r_sf_1.auto_clef()
        l_sf_1.auto_clef()
        l_sf_2.auto_clef()
        r_sf_1.to_stream_voice(2).add_to_score(self.score, part_number=1, staff_number=1)
        r_sf_2.to_stream_voice(1).add_to_score(self.score, part_number=1, staff_number=1)
        l_sf_1.to_stream_voice(2).add_to_score(self.score, part_number=1, staff_number=2)
        l_sf_2.to_stream_voice(1).add_to_score(self.score, part_number=1, staff_number=2)

        xml_path = path + '_test_12.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_13(self):
        # accidental
        r_sf = SimpleFormat(quarter_durations=[4, 4, 4, 4], midis=[61, 63, 64, 67])
        l_sf = SimpleFormat(quarter_durations=[4, 4, 4, 4], midis=[60, 64, 63, 64])
        r_sf.to_stream_voice().add_to_score(self.score, part_number=1, staff_number=1)
        l_sf.to_stream_voice().add_to_score(self.score, part_number=1, staff_number=2)

        xml_path = path + '_test_13.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)


from unittest import TestCase

from musicscore.musictree.midi import Midi
from musicscore.musictree.treechord import TreeChord, TreeNote
from musicscore.musictree.treemeasure import TreeMeasure
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
import os

from musicscore.musicxml.elements.note import Beam

path = os.path.abspath(__file__).split('.')[0]


class TestGrouping(TestCase):
    def setUp(self):
        self.score = TreeScoreTimewise()
        self.score.add_part()

    def test_grouping(self):
        self.make_measure(1, (3, 4))
        self.make_measure(2, (6, 8))

        # for measure in self.score.get_children_by_type(TreeMeasure):
        #     measure.get_part(1).group_beams()

        self.score.finish()

        self.score.write(path=path)

    def make_measure(self, number, time_signature=(4, 4)):
        self.score.add_measure(TreeMeasure(time=time_signature))
        for i in range(time_signature[0] * 8//time_signature[1]):
            self.score.add_chord(number, 1, TreeChord(60, quarter_duration=0.5))



from pathlib import Path

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicxmlunittest import XMLTestCase

path = Path(__file__)


class TestAddTrill(XMLTestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_add_trill(self):
        xml_path = path.parent.joinpath(path.stem + '_add_trill.xml')
        sf = SimpleFormat(quarter_durations=4)
        sf.chords[0].add_trill_mark()
        sf.chords[0].add_wavy_line(type='start')
        sf.chords[0].add_wavy_line(type='stop', relative_x=20)
        sf.to_stream_voice().add_to_score(self.score)
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)



from pathlib import Path

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicscore.musicxml.types.complextypes.defaults import WordFont, LyricFont
from musicxmlunittest import XMLTestCase

path = Path(__file__)


class TestScoreDefaults(XMLTestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_word_font(self):
        sf = SimpleFormat(quarter_durations=[1, 1])
        sf.chords[0].add_words('some very nice words')
        sf.to_stream_voice(1).add_to_score(self.score)
        xml_path = path.parent.joinpath(path.stem + '_word_font.xml')
        self.score.defaults.add_child(WordFont(font_family='Open Sans'))

        self.score.add_title('test title')
        self.score.add_subtitle('test subtitle')
        self.score.write(path=xml_path)
        self.assertCompareFiles(xml_path)

    def test_lyric_font(self):
        sf = SimpleFormat(quarter_durations=[1, 1])
        sf.chords[0].add_lyric('some very nice words')
        sf.to_stream_voice(1).add_to_score(self.score)
        xml_path = path.parent.joinpath(path.stem + '_lyric_font.xml')
        self.score.defaults.add_child(LyricFont(font_family='Open Sans'))

        self.score.add_title('test title')
        self.score.add_subtitle('test subtitle')
        self.score.write(path=xml_path)
        self.assertCompareFiles(xml_path)



import os

from musicscore.musictree.treechord import TreeChord
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicxmlunittest import XMLTestCase

path = str(os.path.abspath(__file__).split('.')[0])


class Test(XMLTestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_1(self):
        self.score.add_measure()
        self.score.add_part()
        self.score.get_measure(1).get_part(1).add_chord(TreeChord(quarter_duration=4, midis=60))
        xml_path = path + '_test_1.xml'
        self.score.write(path=xml_path)
        self.assertCompareFiles(xml_path)

    def test_2(self):
        self.score.add_measure()
        self.score.add_part()
        self.score.get_measure(1).get_part(1).add_chord(TreeChord(quarter_duration=4, midis=60), voice_number=2)
        xml_path = path + '_test_2.xml'
        self.score.write(path=xml_path)
        self.assertCompareFiles(xml_path)

    def test_3(self):
        self.score.add_measure()
        self.score.add_part()
        self.score.get_measure(1).get_part(1).add_chord(TreeChord(quarter_duration=4, midis=70), voice_number=2)
        self.score.get_measure(1).get_part(1).add_chord(TreeChord(quarter_duration=4, midis=60), voice_number=4)
        xml_path = path + '_test_3.xml'
        self.score.write(path=xml_path)
        self.assertCompareFiles(xml_path)

    def test_4(self):
        self.score.add_measure()
        self.score.add_part()
        self.score.get_measure(1).get_part(1).add_chord(TreeChord(quarter_duration=4, midis=70), staff_number=2)
        xml_path = path + '_test_4.xml'
        self.score.write(path=xml_path)
        self.assertCompareFiles(xml_path)

    def test_5(self):
        self.score.add_measure()
        self.score.add_part()
        self.score.get_measure(1).get_part(1).add_chord(TreeChord(quarter_duration=4, midis=60), staff_number=1)
        self.score.get_measure(1).get_part(1).add_chord(TreeChord(quarter_duration=4, midis=70), staff_number=2)
        xml_path = path + '_test_5.xml'
        self.score.write(path=xml_path)
        self.assertCompareFiles(xml_path)


from musicscore.dtd.dtd import Sequence, Element
from musicscore.musicxml.attributes.attribute_abstract import AttributeAbstract
from musicscore.musicxml.attributes.printobject import PrintObject
from musicscore.musicxml.attributes.printspacing import PrintSpacing
from musicscore.musicxml.elements.xml_element import XMLElement
from musicscore.musicxml.types.complextypes.complextype import ComplexType
from musicscore.musicxml.types.simple_type import TypeStaffType, NonNegativeInteger, NonNegativeDecimal, \
    TypeStaffNumber, TypeShowFrets

"""
    <xs:complexType name="staff-details">
        <xs:sequence>
            <xs:element name="staff-type" type="staff-type" minOccurs="0"/>
            <xs:element name="staff-lines" type="xs:nonNegativeInteger" minOccurs="0">
            </xs:element>
            <xs:element name="staff-tuning" type="staff-tuning" minOccurs="0" maxOccurs="unbounded"/>
            <xs:element name="capo" type="xs:nonNegativeInteger" minOccurs="0">
                <xs:annotation>
                    <xs:documentation>
                    </xs:documentation>
                </xs:annotation>
            </xs:element>
            <xs:element name="staff-size" type="non-negative-decimal" minOccurs="0">
                <xs:annotation>
                    <xs:documentation>
                    </xs:documentation>
                </xs:annotation>
            </xs:element>
        </xs:sequence>
        <xs:attribute name="number" type="staff-number"/>
        <xs:attribute name="show-frets" type="show-frets"/>
        <xs:attributeGroup ref="print-object"/>
        <xs:attributeGroup ref="print-spacing"/>
    </xs:complexType>
"""


class StaffType(XMLElement, TypeStaffType):
    """
    """
    _TAG = 'staff-type'

    def __init__(self, value, *args, **kwargs):
        super().__init__(tag=self._TAG, value=value, *args, **kwargs)


class StaffLines(XMLElement, NonNegativeInteger):
    """
    The staff-lines element specifies the number of lines for a non 5-line staff.
    """
    _TAG = 'staff-lines'

    def __init__(self, value, *args, **kwargs):
        super().__init__(tag=self._TAG, value=value, *args, **kwargs)


class StaffTuning(XMLElement):
    """
    """
    _TAG = 'staff-tuning'

    def __init__(self, value, *args, **kwargs):
        super().__init__(tag=self._TAG, value=value, *args, **kwargs)
        raise NotImplementedError()


class Capo(XMLElement, NonNegativeInteger):
    """
    The capo element indicates at which fret a capo should be placed on a fretted instrument. This changes the open
    tuning of the strings specified by staff-tuning by the specified number of half-steps.
    """
    _TAG = 'capo'

    def __init__(self, value, *args, **kwargs):
        super().__init__(tag=self._TAG, value=value, *args, **kwargs)


class StaffSize(XMLElement, NonNegativeDecimal):
    """
    The staff-size element indicates how large a staff space is on this staff, expressed as a percentage of the work's
    default scaling. Values less than 100 make the staff space smaller while values over 100 make the staff space
    larger. A staff-type of cue, ossia, or editorial implies a staff-size of less than 100, but the exact value is
    implementation-dependent unless specified here. Staff size affects staff height only, not the relationship of the
    staff to the left and right margins.
    """
    _TAG = 'staff-size'

    def __init__(self, value, *args, **kwargs):
        super().__init__(tag=self._TAG, value=value, *args, **kwargs)


class ShowFrets(AttributeAbstract):

    def __init__(self, show_frets=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.generate_attribute('show-frets', show_frets, 'TypeShowFrets')
        TypeShowFrets


class ComplexTypeStaffDetails(ComplexType, ShowFrets, PrintObject, PrintSpacing):
    """ The staff-details element is used to indicate different types of staves. The optional number
    attribute specifies the staff number from top to bottom on the system, as with clef. The print-object
    attribute is used to indicate when a staff is not printed in a part, usually in large scores where empty
    parts are omitted. It is yes by default. If print-spacing is yes while print-object is no, the score is
    printed in cutaway format where vertical space is left for the empty part.
    """
    _DTD = Sequence(
        Element(StaffType, min_occurrence=0),
        Element(StaffLines, min_occurrence=0),
        Element(StaffTuning, min_occurrence=0, max_occurrence=None),
        Element(Capo, min_occurrence=0),
        Element(StaffSize, min_occurrence=0),

    )

    def __init__(self, tag, number=1, *args, **kwargs):
        super().__init__(tag=tag, *args, **kwargs)
        self.number = number

    @property
    def number(self):
        return self.get_attribute('number')

    @number.setter
    def number(self, value):
        if value is None:
            self.remove_attribute('number')
        else:
            TypeStaffNumber(value)
            self._ATTRIBUTES.insert(0, 'number')
            self.set_attribute('number', value)



import os

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicxmlunittest import XMLTestCase

path = str(os.path.abspath(__file__).split('.')[0])


class Test(XMLTestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_1(self):
        sf_1 = SimpleFormat(quarter_durations=[2, 2], midis=[0, 71])
        sf_1.chords[1].add_words('up')
        sf_1.to_stream_voice().add_to_score(self.score, staff_number=1)

        sf_2 = SimpleFormat(quarter_durations=[2, 2], midis=[71, 0])
        sf_2.chords[0].add_words('down')
        sf_2.to_stream_voice().add_to_score(self.score, staff_number=2)

        xml_path = path + '_test_1.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_2(self):
        sf_1 = SimpleFormat(quarter_durations=[2, 2], midis=[0, 71])
        sf_1.chords[1].add_words('up')
        sf_1.to_stream_voice(1).add_to_score(self.score, staff_number=1)
        sf_2 = SimpleFormat(quarter_durations=[2, 2], midis=[60, 0])
        sf_2.chords[0].add_words('down', placement='below')
        sf_2.to_stream_voice(2).add_to_score(self.score, staff_number=1)

        sf_1 = SimpleFormat(quarter_durations=[2, 2], midis=[0, 71])
        sf_1.chords[1].add_words('up')
        sf_1.to_stream_voice(1).add_to_score(self.score, staff_number=2)
        sf_2 = SimpleFormat(quarter_durations=[2, 2], midis=[60, 0])
        sf_2.chords[0].add_words('down', placement='below')
        sf_2.to_stream_voice(2).add_to_score(self.score, staff_number=2)

        xml_path = path + '_test_2.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)



import os

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicxmlunittest import XMLTestCase

path = os.path.abspath(__file__).split('.')[0]


class Test(XMLTestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_1(self):
        sf = SimpleFormat(quarter_durations=[7])

        sf.chords[0].add_tremolo()

        sf.to_stream_voice().add_to_score(self.score)
        xml_path = path + '_test_1.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    # def test_2(self):
    #     sf = SimpleFormat(quarter_durations=[7])
    #
    #     sf.chords[0].add_tremolo(number=4)
    #
    #     sf.to_stream_voice().add_to_score(self.score)
    #     xml_path = path + '_test_2.xml'
    #     self.score.write(xml_path)
    #     self.assertCompareFiles(xml_path)

    def test_3(self):
        sf = SimpleFormat(quarter_durations=[7], midis=[(60, 64)])

        sf.chords[0].add_tremolo(3)

        sf.to_stream_voice().add_to_score(self.score)
        xml_path = path + '_test_3.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)
